/**
 * A Low Energy radio stack for NRF microcontrollers.
 *
 * @author  Ehsan Tofigh
 * @project lcx
 * @file    include/libradio/ale.h
 * @license
 */

#pragma once

/* Use C linkage to avoid name mangling in C++: */
#ifdef __cplusplus
extern "C" {
#endif

#include <libcortex/ccm.h>

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

/** Configuration structure for the ALE module. */
struct lrd_ale_handle {
	uint8_t key[16U];       /**< 16 byte AES key. */
	uint8_t iv[8U];         /**< 8 byte initialisation vector. */
};

/** Structure representing a 6 octet MAC address. */
union lrd_ale_mac {
	uint8_t raw[6U];
	struct {
		uint32_t low;
		uint16_t high;
	} __attribute__((packed));
};

/** List of frame types. */
enum lrd_ale_frame_type {
	LRD_ALE_FRAME_TYPE_BEACON,
	LRD_ALE_FRAME_TYPE_DATA,

	LRD_ALE_FRAME_TYPE_COUNT
};

/** Structure to represent a packet header. */
struct lrd_ale_header {
	struct {
		uint8_t type    : 4;
		uint8_t         : 2;
		uint8_t tx_addr : 1;
		uint8_t rx_addr : 1;
	};
	uint8_t length;
	uint8_t dummy;
} __attribute__((packed));

/**
 * Initialise the radio peripheral.
 *
 * @param[in] handle Pointer to the configuration structure.
 *
 * @retval true  Successfully initialised the radio peripheral.
 * @retval false Failed to initialise the radio peripheral.
 */
extern bool lrd_ale_init (const struct lrd_ale_handle *handle);

/**
 * Start listening for packets on the radio.
 *
 * @retval true  Successfully started the receiver.
 * @retval false Failed to start the receiver.
 */
extern bool lrd_ale_start_receiver (void);

/**
 * Start measuring the energy levels and return the maximum level.
 *
 * @param[in] cnt The number of measurements to take.
 *
 * @return The maximum energy level detected during the measurement window.
 */
extern uint8_t lrd_ale_read_edlevel (const uint32_t cnt);

/**
 * Transmit a packet using the radio.
 *
 * @param[in,out] header Pointer to the packet's header structure.
 * @param[in]     len    Size of the full packet in bytes.
 * @param[in]     cca    If true, perform clear channel assessment before
 *                       transmission.
 *
 * @retval true  Successfully initiated the transmission.
 * @retval false Failed to initiate the transmission.
 */
extern bool lrd_ale_transmit (struct lrd_ale_header *header,
			      const size_t           len,
			      const bool             cca);

/* Close C linkage directive for C++: */
#ifdef __cplusplus
}
#endif
